@@ -36,6 +36,8 @@ class Denoise(Protocol):
 
 class IDiffusion:
     _get_cond: Callable
+    predict_eps_from_z_and_v: Callable
+    predict_start_from_z_and_v: Callable
 
     denoise: Denoise
     q_sampler: "DDPMQSampler"
@@ -99,6 +101,7 @@ class ISampler(WithRegister):
 
     def __init__(self, model: IDiffusion):
         self.model = model
+        self.initialized = False
 
     @property
     @abstractmethod
@@ -163,6 +166,7 @@ class ISampler(WithRegister):
                 ref_ts = get_timesteps(num_steps - step - 1, ref.shape[0], z.device)
                 ref_noisy = self.q_sample(ref, ref_ts, ref_noise)
                 image = ref_noisy * ref_mask + image * (1.0 - ref_mask)
+        self.initialized = False
         return image
 
 
