@@ -6,20 +6,19 @@ from torch import Tensor
 from typing import Any
 from typing import Dict
 from typing import Optional
+from cftool.types import tensor_dict_type
 
 from ..general import EncoderDecoder
-from ..protocol import GaussianGeneratorMixin
-from ....types import tensor_dict_type
-from ....protocol import ModelProtocol
-from ....protocol import TrainerState
 from ....constants import INPUT_KEY
 from ....constants import LABEL_KEY
 from ....constants import PREDICTIONS_KEY
+from ...protocols.cv import GaussianGeneratorMixin
 from ....losses.vae import MU_KEY
 from ....losses.vae import LOG_VAR_KEY
 from ....modules.blocks import Lambda
 from ....modules.blocks import Activation
 from ....modules.blocks import ChannelPadding
+from ....misc.internal_.register import register_module
 
 
 def reparameterize(mu: Tensor, log_var: Tensor) -> Tensor:
@@ -28,7 +27,7 @@ def reparameterize(mu: Tensor, log_var: Tensor) -> Tensor:
     return eps * std + mu
 
 
-class VanillaVAEBase(ModelProtocol, GaussianGeneratorMixin):
+class VanillaVAEBase(nn.Module, GaussianGeneratorMixin):
     from_latent: nn.Module
 
     def __init__(
@@ -105,13 +104,7 @@ class VanillaVAEBase(ModelProtocol, GaussianGeneratorMixin):
             net = self.out(net)
         return net
 
-    def forward(
-        self,
-        batch_idx: int,
-        batch: tensor_dict_type,
-        state: Optional[TrainerState] = None,
-        **kwargs: Any,
-    ) -> tensor_dict_type:
+    def forward(self, batch: tensor_dict_type, **kwargs: Any) -> tensor_dict_type:
         net = self.generator.encode(batch, **kwargs)
         mu, log_var = net.chunk(2, dim=1)
         net = reparameterize(mu, log_var)
@@ -120,8 +113,8 @@ class VanillaVAEBase(ModelProtocol, GaussianGeneratorMixin):
         return {PREDICTIONS_KEY: net, MU_KEY: mu, LOG_VAR_KEY: log_var}
 
 
-@ModelProtocol.register("vae")
-@ModelProtocol.register("vae1d")
+@register_module("vae")
+@register_module("vae1d")
 class VanillaVAE1D(VanillaVAEBase):
     def __init__(
         self,
@@ -143,10 +136,6 @@ class VanillaVAE1D(VanillaVAEBase):
         decoder_config: Optional[Dict[str, Any]] = None,
         output_activation: Optional[str] = "tanh",
     ):
-        if decoder_config is None:
-            decoder_config = {}
-        if decoder == "vanilla":
-            decoder_config["latent_expand_ratio"] = 2
         super().__init__(
             True,
             in_channels,
@@ -169,7 +158,7 @@ class VanillaVAE1D(VanillaVAEBase):
         self.from_latent = nn.Identity()
 
 
-@ModelProtocol.register("vae2d")
+@register_module("vae2d")
 class VanillaVAE2D(VanillaVAEBase):
     def __init__(
         self,
